from serpent.machine_learning.context_classification.context_classifiers.svm_context_classifier import SVMContextClassifier
from serpent.machine_learning.context_classification.context_classifiers.cnn_inception_v3_context_classifier import CNNInceptionV3ContextClassifier

